      SUBROUTINE DEZFTB(N,R,AZERO,A,B,WSAVE) 
C***BEGIN PROLOGUE  DEZFTB    
C***DATE WRITTEN   790601   (YYMMDD)    
C***REVISION DATE  860115   (YYMMDD)    
C***CATEGORY NO.  J1A1   
C***KEYWORDS  FOURIER TRANSFORM    
C***AUTHOR  SWARZTRAUBER, P. N., (NCAR) 
C***PURPOSE  A Simplified double precision, periodic, 
C              backward transform
C***DESCRIPTION
C           From the book, "Numerical Methods and Software" by
C                D. Kahaner, C. Moler, S. Nash
C                Prentice Hall, 1988
C    
C  Subroutine DEZFTB computes a d. p. periodic sequence from its   
C  Fourier coefficients (Fourier synthesis).  The transform is   
C  defined below at Output Parameter R.  DEZFTB is a simplified  
C  but slower version of DRFFTB.    
C    
C  Input Parameters 
C    
C  N       the length of the output array R.  The method is most 
C          efficient when N is the product of small primes. 
C    
C  AZERO   the constant Fourier coefficient  
C    
C  A,B     arrays which contain the remaining Fourier coefficients.   
C          These arrays are not destroyed.   
C    
C          The length of these arrays depends on whether N is even or 
C          odd.
C    
C          If N is even, N/2    locations are required.
C          If N is odd, (N-1)/2 locations are required 
C    
C  WSAVE   a work array which must be dimensioned at least 3*N+15
C          in the program that calls DEZFTB.  The WSAVE array must be 
C          initialized by calling subroutine DEZFTI(N,WSAVE), and a   
C          different WSAVE array must be used for each different 
C          value of N.  This initialization does not have to be  
C          repeated so long as N remains unchanged.  Thus subsequent  
C          transforms can be obtained faster than the first.
C          The same WSAVE array can be used by EZFFTF and DEZFTB.
C    
C    
C  Output Parameters
C    
C  R       if N is even, define KMAX=N/2
C          if N is odd,  define KMAX=(N-1)/2 
C    
C          Then for I=1,...,N 
C    
C               R(I)=AZERO plus the sum from K=1 to K=KMAX of    
C    
C               A(K)*COS(K*(I-1)*2*PI/N)+B(K)*SIN(K*(I-1)*2*PI/N)
C    
C  ********************* Complex Notation **************************  
C    
C          For J=1,...,N 
C    
C          R(J) equals the sum from K=-KMAX to K=KMAX of    
C    
C               C(K)*EXP(I*K*(J-1)*2*PI/N)   
C    
C          where    
C    
C               C(K) = .5D0*CMPLX(A(K),-B(K))   for K=1,...,KMAX   
C    
C               C(-K) = CONJG(C(K))
C    
C               C(0) = AZERO  
C    
C                    and I=SQRT(-1)
C    
C  *************** Amplitude - Phase Notation *********************** 
C    
C          For I=1,...,N 
C    
C          R(I) equals AZERO plus the sum from K=1 to K=KMAX of  
C    
C               ALPHA(K)*COS(K*(I-1)*2*PI/N+BETA(K))   
C    
C          where    
C    
C               ALPHA(K) = SQRT(A(K)*A(K)+B(K)*B(K))   
C    
C               COS(BETA(K))=A(K)/ALPHA(K)   
C    
C               SIN(BETA(K))=-B(K)/ALPHA(K)  
C    
C  *                                                                   *   
C  *   References                                                      *   
C  *                                                                   *   
C  *   1. P.N. Swarztrauber, Vectorizing the FFTs, in Parallel         *   
C  *      Computations (G. Rodrigue, ed.), Academic Press, 1982,       *   
C  *      pp. 51-83.                                                   *   
C  *   2. B.L. Buzbee, The SLATEC Common Math Library, in Sources      *   
C  *      and Development of Mathematical Software (W. Cowell, ed.),   *   
C  *      Prentice-Hall, 1984, pp. 302-318.                            *   
C  *                                                                   *   
C  *********************************************************************   
C    
C***REFERENCES  (NONE)   
C***ROUTINES CALLED  DRFFTB    
C***END PROLOGUE  DEZFTB 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION       R(*)       ,A(*)       ,B(*)       ,WSAVE(*)    
C***FIRST EXECUTABLE STATEMENT  DEZFTB  
      IF (N-2) 101,102,103    
  101 R(1) = AZERO  
      RETURN   
  102 R(1) = AZERO+A(1)  
      R(2) = AZERO-A(1)  
      RETURN   
  103 NS2 = (N-1)/2 
      DO 104 I=1,NS2
         R(2*I) = .5D0*A(I)
         R(2*I+1) = -.5D0*B(I)  
  104 CONTINUE 
      R(1) = AZERO  
      IF (MOD(N,2) .EQ. 0) R(N) = A(NS2+1)   
      CALL DRFFTB (N,R,WSAVE(N+1))  
      RETURN   
      END 
      SUBROUTINE DRFFTB(N,R,WSAVE)  
C***BEGIN PROLOGUE  DRFFTB
C***DATE WRITTEN   790601   (YYMMDD)    
C***REVISION DATE  890413   (YYMMDD)    
C***CATEGORY NO.  J1A1   
C***KEYWORDS  FOURIER TRANSFORM    
C***AUTHOR  SWARZTRAUBER, P. N., (NCAR) 
C***PURPOSE  Backward transform of a dp coefficient array.     
C***DESCRIPTION
C     From the book "Numerical Methods and Software"
C          by  D. Kahaner, C. Moler, S. Nash
C               Prentice Hall 1989
C    
C  Subroutine DRFFTB computes the dp perodic sequence from its  
C  Fourier coefficients (Fourier synthesis).  The transform is defined
C  below at output parameter R.    
C    
C  Input Parameters 
C    
C  N       the length of the array R to be transformed.  The method   
C          is most efficient when N is a product of small primes.
C          N may change so long as different work arrays are provided.
C    
C  R       a dp array of length N which contains the sequence  
C          to be transformed  
C    
C  WSAVE   a dp work array which must be dimensioned at least 2*N+15
C          in the program that calls DRFFTB.  The WSAVE array must be  
C          initialized by calling subroutine DRFFTI(N,WSAVE), and a    
C          different WSAVE array must be used for each different 
C          value of N.  This initialization does not have to be  
C          repeated so long as N remains unchanged.  Thus subsequent  
C          transforms can be obtained faster than the first.
C          The same WSAVE array can be used by DRFFTF and DRFFTB.  
C    
C    
C  Output Parameters
C    
C  R       For N even and For I = 1,...,N    
C    
C               R(I) = R(1)+(-1)**(I-1)*R(N) 
C    
C                    plus the sum from K=2 to K=N/2 of 
C    
C                     2.*R(2*K-2)*COS((K-1)*(I-1)*2*PI/N)   
C    
C                    -2.*R(2*K-1)*SIN((K-1)*(I-1)*2*PI/N)   
C    
C          For N odd and For I = 1,...,N
C    
C               R(I) = R(1) plus the sum from K=2 to K=(N+1)/2 of
C    
C                    2.*R(2*K-2)*COS((K-1)*(I-1)*2*PI/N)    
C    
C                   -2.*R(2*K-1)*SIN((K-1)*(I-1)*2*PI/N)    
C    
C   *****  Note:    
C               This transform is unnormalized since a call of DRFFTF  
C               followed by a call of DRFFTB will multiply the input   
C               sequence by N.
C    
C  WSAVE   contains results which must not be destroyed between  
C          calls of DRFFTB or DRFFTF.
C    
C  *                                                                   *   
C  *   1. P.N. Swarztrauber, Vectorizing the FFTs, in Parallel         *   
C  *      Computations (G. Rodrigue, ed.), Academic Press, 1982,       *   
C  *      pp. 51-83.                                                   *   
C  *   2. B.L. Buzbee, The SLATEC Common Math Library, in Sources      *   
C  *      and Development of Mathematical Software (W. Cowell, ed.),   *   
C  *      Prentice-Hall, 1984, pp. 302-318.                            *   
C  *                                                                   *   
C  *********************************************************************   
C    
C***REFERENCES  (NONE)   
C***ROUTINES CALLED  DRFTB1   
C***END PROLOGUE  DRFFTB  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z) 
      DIMENSION       R(*)       ,WSAVE(*)   
C***FIRST EXECUTABLE STATEMENT  DRFFTB   
      IF (N .EQ. 1) RETURN    
      CALL DRFTB1 (N,R,WSAVE,WSAVE(N+1),WSAVE(2*N+1))  
      RETURN   
      END
      SUBROUTINE DRFTB1(N,C,CH,WA,IFAC) 
C***BEGIN PROLOGUE  DRFTB1    
C***REFER TO  DRFFTB 
C***ROUTINES CALLED  DRADB2,DRADB3,DRADB4,DRADB5,DRADBG
C***END PROLOGUE  DRFTB1 
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION       CH(*)      ,C(*)       ,WA(*)      ,IFAC(*)
C***FIRST EXECUTABLE STATEMENT  DRFTB1  
      NF = IFAC(2)  
      NA = 0   
      L1 = 1   
      IW = 1   
      DO 116 K1=1,NF
         IP = IFAC(K1+2) 
         L2 = IP*L1 
         IDO = N/L2 
         IDL1 = IDO*L1   
         IF (IP .NE. 4) GO TO 103  
         IX2 = IW+IDO    
         IX3 = IX2+IDO   
         IF (NA .NE. 0) GO TO 101  
         CALL DRADB4 (IDO,L1,C,CH,WA(IW),WA(IX2),WA(IX3))    
         GO TO 102  
  101    CALL DRADB4 (IDO,L1,CH,C,WA(IW),WA(IX2),WA(IX3))    
  102    NA = 1-NA  
         GO TO 115  
  103    IF (IP .NE. 2) GO TO 106  
         IF (NA .NE. 0) GO TO 104  
         CALL DRADB2 (IDO,L1,C,CH,WA(IW))
         GO TO 105  
  104    CALL DRADB2 (IDO,L1,CH,C,WA(IW))
  105    NA = 1-NA  
         GO TO 115  
  106    IF (IP .NE. 3) GO TO 109  
         IX2 = IW+IDO    
         IF (NA .NE. 0) GO TO 107  
         CALL DRADB3 (IDO,L1,C,CH,WA(IW),WA(IX2))  
         GO TO 108  
  107    CALL DRADB3 (IDO,L1,CH,C,WA(IW),WA(IX2))  
  108    NA = 1-NA  
         GO TO 115  
  109    IF (IP .NE. 5) GO TO 112  
         IX2 = IW+IDO    
         IX3 = IX2+IDO   
         IX4 = IX3+IDO   
         IF (NA .NE. 0) GO TO 110  
         CALL DRADB5 (IDO,L1,C,CH,WA(IW),WA(IX2),WA(IX3),WA(IX4)) 
         GO TO 111  
  110    CALL DRADB5 (IDO,L1,CH,C,WA(IW),WA(IX2),WA(IX3),WA(IX4)) 
  111    NA = 1-NA  
         GO TO 115  
  112    IF (NA .NE. 0) GO TO 113  
         CALL DRADBG (IDO,IP,L1,IDL1,C,C,C,CH,CH,WA(IW))
         GO TO 114  
  113    CALL DRADBG (IDO,IP,L1,IDL1,CH,CH,CH,C,C,WA(IW))    
  114    IF (IDO .EQ. 1) NA = 1-NA 
  115    L1 = L2    
         IW = IW+(IP-1)*IDO   
  116 CONTINUE 
      IF (NA .EQ. 0) RETURN   
      DO 117 I=1,N  
         C(I) = CH(I)    
  117 CONTINUE 
      RETURN   
      END 
      SUBROUTINE DRADB2(IDO,L1,CC,CH,WA1)
C***BEGIN PROLOGUE  DRADB2
C***REFER TO  DRFFTB 
C***ROUTINES CALLED  (NONE)   
C***END PROLOGUE  DRADB2  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION       CC(IDO,2,L1)           ,CH(IDO,L1,2)           ,
     1                WA1(*)  
C***FIRST EXECUTABLE STATEMENT  DRADB2   
      DO 101 K=1,L1 
         CH(1,K,1) = CC(1,1,K)+CC(IDO,2,K)   
         CH(1,K,2) = CC(1,1,K)-CC(IDO,2,K)   
  101 CONTINUE 
      IF (IDO-2) 107,105,102  
  102 IDP2 = IDO+2  
      IF((IDO-1)/2.LT.L1) GO TO 108
      DO 104 K=1,L1 
CDIR$ IVDEP    
         DO 103 I=3,IDO,2
            IC = IDP2-I  
            CH(I-1,K,1) = CC(I-1,1,K)+CC(IC-1,2,K)
            TR2 = CC(I-1,1,K)-CC(IC-1,2,K)   
            CH(I,K,1) = CC(I,1,K)-CC(IC,2,K) 
            TI2 = CC(I,1,K)+CC(IC,2,K)  
            CH(I-1,K,2) = WA1(I-2)*TR2-WA1(I-1)*TI2    
            CH(I,K,2) = WA1(I-2)*TI2+WA1(I-1)*TR2 
  103    CONTINUE   
  104 CONTINUE 
      GO TO 111
  108 DO 110 I=3,IDO,2   
         IC = IDP2-I
CDIR$ IVDEP    
         DO 109 K=1,L1   
            CH(I-1,K,1) = CC(I-1,1,K)+CC(IC-1,2,K)
            TR2 = CC(I-1,1,K)-CC(IC-1,2,K)   
            CH(I,K,1) = CC(I,1,K)-CC(IC,2,K) 
            TI2 = CC(I,1,K)+CC(IC,2,K)  
            CH(I-1,K,2) = WA1(I-2)*TR2-WA1(I-1)*TI2    
            CH(I,K,2) = WA1(I-2)*TI2+WA1(I-1)*TR2 
  109    CONTINUE   
  110 CONTINUE 
  111 IF (MOD(IDO,2) .EQ. 1) RETURN
  105 DO 106 K=1,L1 
         CH(IDO,K,1) = CC(IDO,1,K)+CC(IDO,1,K)    
         CH(IDO,K,2) = -(CC(1,2,K)+CC(1,2,K))
  106 CONTINUE 
  107 RETURN   
      END 
      SUBROUTINE DRADB3(IDO,L1,CC,CH,WA1,WA2) 
C***BEGIN PROLOGUE  DRADB3
C***REFER TO  DRFFTB 
C***ROUTINES CALLED  (NONE)   
C***END PROLOGUE  DRADB3  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION       CC(IDO,3,L1)           ,CH(IDO,L1,3)           ,
     1                WA1(*)     ,WA2(*)
C***FIRST EXECUTABLE STATEMENT  DRADB3   
      TAUR = -.5D0    
      TAUI = .5D0*SQRT(3.0D0) 
      DO 101 K=1,L1 
         TR2 = CC(IDO,2,K)+CC(IDO,2,K)  
         CR2 = CC(1,1,K)+TAUR*TR2  
         CH(1,K,1) = CC(1,1,K)+TR2 
         CI3 = TAUI*(CC(1,3,K)+CC(1,3,K))    
         CH(1,K,2) = CR2-CI3  
         CH(1,K,3) = CR2+CI3  
  101 CONTINUE 
      IF (IDO .EQ. 1) RETURN  
      IDP2 = IDO+2  
      IF((IDO-1)/2.LT.L1) GO TO 104
      DO 103 K=1,L1 
CDIR$ IVDEP    
         DO 102 I=3,IDO,2
            IC = IDP2-I  
            TR2 = CC(I-1,3,K)+CC(IC-1,2,K)   
            CR2 = CC(I-1,1,K)+TAUR*TR2  
            CH(I-1,K,1) = CC(I-1,1,K)+TR2    
            TI2 = CC(I,3,K)-CC(IC,2,K)  
            CI2 = CC(I,1,K)+TAUR*TI2    
            CH(I,K,1) = CC(I,1,K)+TI2   
            CR3 = TAUI*(CC(I-1,3,K)-CC(IC-1,2,K)) 
            CI3 = TAUI*(CC(I,3,K)+CC(IC,2,K))
            DR2 = CR2-CI3
            DR3 = CR2+CI3
            DI2 = CI2+CR3
            DI3 = CI2-CR3
            CH(I-1,K,2) = WA1(I-2)*DR2-WA1(I-1)*DI2    
            CH(I,K,2) = WA1(I-2)*DI2+WA1(I-1)*DR2 
            CH(I-1,K,3) = WA2(I-2)*DR3-WA2(I-1)*DI3    
            CH(I,K,3) = WA2(I-2)*DI3+WA2(I-1)*DR3 
  102    CONTINUE   
  103 CONTINUE 
      RETURN   
  104 DO 106 I=3,IDO,2   
         IC = IDP2-I
CDIR$ IVDEP    
         DO 105 K=1,L1   
            TR2 = CC(I-1,3,K)+CC(IC-1,2,K)   
            CR2 = CC(I-1,1,K)+TAUR*TR2  
            CH(I-1,K,1) = CC(I-1,1,K)+TR2    
            TI2 = CC(I,3,K)-CC(IC,2,K)  
            CI2 = CC(I,1,K)+TAUR*TI2    
            CH(I,K,1) = CC(I,1,K)+TI2   
            CR3 = TAUI*(CC(I-1,3,K)-CC(IC-1,2,K)) 
            CI3 = TAUI*(CC(I,3,K)+CC(IC,2,K))
            DR2 = CR2-CI3
            DR3 = CR2+CI3
            DI2 = CI2+CR3
            DI3 = CI2-CR3
            CH(I-1,K,2) = WA1(I-2)*DR2-WA1(I-1)*DI2    
            CH(I,K,2) = WA1(I-2)*DI2+WA1(I-1)*DR2 
            CH(I-1,K,3) = WA2(I-2)*DR3-WA2(I-1)*DI3    
            CH(I,K,3) = WA2(I-2)*DI3+WA2(I-1)*DR3 
  105    CONTINUE   
  106 CONTINUE 
      RETURN   
      END 
      SUBROUTINE DRADB4(IDO,L1,CC,CH,WA1,WA2,WA3)  
C***BEGIN PROLOGUE  DRADB4
C***REFER TO  DRFFTB 
C***ROUTINES CALLED  (NONE)   
C***END PROLOGUE  DRADB4  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION       CC(IDO,4,L1)           ,CH(IDO,L1,4)           ,
     1                WA1(*)     ,WA2(*)     ,WA3(*)   
C***FIRST EXECUTABLE STATEMENT  DRADB4   
      SQRT2 = SQRT(2.0D0)   
      DO 101 K=1,L1 
         TR1 = CC(1,1,K)-CC(IDO,4,K)    
         TR2 = CC(1,1,K)+CC(IDO,4,K)    
         TR3 = CC(IDO,2,K)+CC(IDO,2,K)  
         TR4 = CC(1,3,K)+CC(1,3,K) 
         CH(1,K,1) = TR2+TR3  
         CH(1,K,2) = TR1-TR4  
         CH(1,K,3) = TR2-TR3  
         CH(1,K,4) = TR1+TR4  
  101 CONTINUE 
      IF (IDO-2) 107,105,102  
  102 IDP2 = IDO+2  
      IF((IDO-1)/2.LT.L1) GO TO 108
      DO 104 K=1,L1 
CDIR$ IVDEP    
         DO 103 I=3,IDO,2
            IC = IDP2-I  
            TI1 = CC(I,1,K)+CC(IC,4,K)  
            TI2 = CC(I,1,K)-CC(IC,4,K)  
            TI3 = CC(I,3,K)-CC(IC,2,K)  
            TR4 = CC(I,3,K)+CC(IC,2,K)  
            TR1 = CC(I-1,1,K)-CC(IC-1,4,K)   
            TR2 = CC(I-1,1,K)+CC(IC-1,4,K)   
            TI4 = CC(I-1,3,K)-CC(IC-1,2,K)   
            TR3 = CC(I-1,3,K)+CC(IC-1,2,K)   
            CH(I-1,K,1) = TR2+TR3  
            CR3 = TR2-TR3
            CH(I,K,1) = TI2+TI3    
            CI3 = TI2-TI3
            CR2 = TR1-TR4
            CR4 = TR1+TR4
            CI2 = TI1+TI4
            CI4 = TI1-TI4
            CH(I-1,K,2) = WA1(I-2)*CR2-WA1(I-1)*CI2    
            CH(I,K,2) = WA1(I-2)*CI2+WA1(I-1)*CR2 
            CH(I-1,K,3) = WA2(I-2)*CR3-WA2(I-1)*CI3    
            CH(I,K,3) = WA2(I-2)*CI3+WA2(I-1)*CR3 
            CH(I-1,K,4) = WA3(I-2)*CR4-WA3(I-1)*CI4    
            CH(I,K,4) = WA3(I-2)*CI4+WA3(I-1)*CR4 
  103    CONTINUE   
  104 CONTINUE 
      GO TO 111
  108 DO 110 I=3,IDO,2   
         IC = IDP2-I
CDIR$ IVDEP    
         DO 109 K=1,L1   
            TI1 = CC(I,1,K)+CC(IC,4,K)  
            TI2 = CC(I,1,K)-CC(IC,4,K)  
            TI3 = CC(I,3,K)-CC(IC,2,K)  
            TR4 = CC(I,3,K)+CC(IC,2,K)  
            TR1 = CC(I-1,1,K)-CC(IC-1,4,K)   
            TR2 = CC(I-1,1,K)+CC(IC-1,4,K)   
            TI4 = CC(I-1,3,K)-CC(IC-1,2,K)   
            TR3 = CC(I-1,3,K)+CC(IC-1,2,K)   
            CH(I-1,K,1) = TR2+TR3  
            CR3 = TR2-TR3
            CH(I,K,1) = TI2+TI3    
            CI3 = TI2-TI3
            CR2 = TR1-TR4
            CR4 = TR1+TR4
            CI2 = TI1+TI4
            CI4 = TI1-TI4
            CH(I-1,K,2) = WA1(I-2)*CR2-WA1(I-1)*CI2    
            CH(I,K,2) = WA1(I-2)*CI2+WA1(I-1)*CR2 
            CH(I-1,K,3) = WA2(I-2)*CR3-WA2(I-1)*CI3    
            CH(I,K,3) = WA2(I-2)*CI3+WA2(I-1)*CR3 
            CH(I-1,K,4) = WA3(I-2)*CR4-WA3(I-1)*CI4    
            CH(I,K,4) = WA3(I-2)*CI4+WA3(I-1)*CR4 
  109    CONTINUE   
  110 CONTINUE 
  111 IF (MOD(IDO,2) .EQ. 1) RETURN
  105 DO 106 K=1,L1 
         TI1 = CC(1,2,K)+CC(1,4,K) 
         TI2 = CC(1,4,K)-CC(1,2,K) 
         TR1 = CC(IDO,1,K)-CC(IDO,3,K)  
         TR2 = CC(IDO,1,K)+CC(IDO,3,K)  
         CH(IDO,K,1) = TR2+TR2
         CH(IDO,K,2) = SQRT2*(TR1-TI1)  
         CH(IDO,K,3) = TI2+TI2
         CH(IDO,K,4) = -SQRT2*(TR1+TI1) 
  106 CONTINUE 
  107 RETURN   
      END 
      SUBROUTINE DRADB5(IDO,L1,CC,CH,WA1,WA2,WA3,WA4)   
C***BEGIN PROLOGUE  DRADB5
C***REFER TO  DRFFTB 
C***ROUTINES CALLED  (NONE)   
C***END PROLOGUE  DRADB5  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION       CC(IDO,5,L1)           ,CH(IDO,L1,5)           ,
     1                WA1(*)     ,WA2(*)     ,WA3(*)     ,WA4(*) 
C***FIRST EXECUTABLE STATEMENT  DRADB5   
      PI = 4.0D0*ATAN(1.0D0)   
      TR11 = SIN(.1D0*PI)  
      TI11 = SIN(.4D0*PI)  
      TR12 = -SIN(.3D0*PI) 
      TI12 = SIN(.2D0*PI)  
      DO 101 K=1,L1 
         TI5 = CC(1,3,K)+CC(1,3,K) 
         TI4 = CC(1,5,K)+CC(1,5,K) 
         TR2 = CC(IDO,2,K)+CC(IDO,2,K)  
         TR3 = CC(IDO,4,K)+CC(IDO,4,K)  
         CH(1,K,1) = CC(1,1,K)+TR2+TR3  
         CR2 = CC(1,1,K)+TR11*TR2+TR12*TR3   
         CR3 = CC(1,1,K)+TR12*TR2+TR11*TR3   
         CI5 = TI11*TI5+TI12*TI4   
         CI4 = TI12*TI5-TI11*TI4   
         CH(1,K,2) = CR2-CI5  
         CH(1,K,3) = CR3-CI4  
         CH(1,K,4) = CR3+CI4  
         CH(1,K,5) = CR2+CI5  
  101 CONTINUE 
      IF (IDO .EQ. 1) RETURN  
      IDP2 = IDO+2  
      IF((IDO-1)/2.LT.L1) GO TO 104
      DO 103 K=1,L1 
CDIR$ IVDEP    
         DO 102 I=3,IDO,2
            IC = IDP2-I  
            TI5 = CC(I,3,K)+CC(IC,2,K)  
            TI2 = CC(I,3,K)-CC(IC,2,K)  
            TI4 = CC(I,5,K)+CC(IC,4,K)  
            TI3 = CC(I,5,K)-CC(IC,4,K)  
            TR5 = CC(I-1,3,K)-CC(IC-1,2,K)   
            TR2 = CC(I-1,3,K)+CC(IC-1,2,K)   
            TR4 = CC(I-1,5,K)-CC(IC-1,4,K)   
            TR3 = CC(I-1,5,K)+CC(IC-1,4,K)   
            CH(I-1,K,1) = CC(I-1,1,K)+TR2+TR3
            CH(I,K,1) = CC(I,1,K)+TI2+TI3    
            CR2 = CC(I-1,1,K)+TR11*TR2+TR12*TR3   
            CI2 = CC(I,1,K)+TR11*TI2+TR12*TI3
            CR3 = CC(I-1,1,K)+TR12*TR2+TR11*TR3   
            CI3 = CC(I,1,K)+TR12*TI2+TR11*TI3
            CR5 = TI11*TR5+TI12*TR4
            CI5 = TI11*TI5+TI12*TI4
            CR4 = TI12*TR5-TI11*TR4
            CI4 = TI12*TI5-TI11*TI4
            DR3 = CR3-CI4
            DR4 = CR3+CI4
            DI3 = CI3+CR4
            DI4 = CI3-CR4
            DR5 = CR2+CI5
            DR2 = CR2-CI5
            DI5 = CI2-CR5
            DI2 = CI2+CR5
            CH(I-1,K,2) = WA1(I-2)*DR2-WA1(I-1)*DI2    
            CH(I,K,2) = WA1(I-2)*DI2+WA1(I-1)*DR2 
            CH(I-1,K,3) = WA2(I-2)*DR3-WA2(I-1)*DI3    
            CH(I,K,3) = WA2(I-2)*DI3+WA2(I-1)*DR3 
            CH(I-1,K,4) = WA3(I-2)*DR4-WA3(I-1)*DI4    
            CH(I,K,4) = WA3(I-2)*DI4+WA3(I-1)*DR4 
            CH(I-1,K,5) = WA4(I-2)*DR5-WA4(I-1)*DI5    
            CH(I,K,5) = WA4(I-2)*DI5+WA4(I-1)*DR5 
  102    CONTINUE   
  103 CONTINUE 
      RETURN   
  104 DO 106 I=3,IDO,2   
         IC = IDP2-I
CDIR$ IVDEP    
         DO 105 K=1,L1   
            TI5 = CC(I,3,K)+CC(IC,2,K)  
            TI2 = CC(I,3,K)-CC(IC,2,K)  
            TI4 = CC(I,5,K)+CC(IC,4,K)  
            TI3 = CC(I,5,K)-CC(IC,4,K)  
            TR5 = CC(I-1,3,K)-CC(IC-1,2,K)   
            TR2 = CC(I-1,3,K)+CC(IC-1,2,K)   
            TR4 = CC(I-1,5,K)-CC(IC-1,4,K)   
            TR3 = CC(I-1,5,K)+CC(IC-1,4,K)   
            CH(I-1,K,1) = CC(I-1,1,K)+TR2+TR3
            CH(I,K,1) = CC(I,1,K)+TI2+TI3    
            CR2 = CC(I-1,1,K)+TR11*TR2+TR12*TR3   
            CI2 = CC(I,1,K)+TR11*TI2+TR12*TI3
            CR3 = CC(I-1,1,K)+TR12*TR2+TR11*TR3   
            CI3 = CC(I,1,K)+TR12*TI2+TR11*TI3
            CR5 = TI11*TR5+TI12*TR4
            CI5 = TI11*TI5+TI12*TI4
            CR4 = TI12*TR5-TI11*TR4
            CI4 = TI12*TI5-TI11*TI4
            DR3 = CR3-CI4
            DR4 = CR3+CI4
            DI3 = CI3+CR4
            DI4 = CI3-CR4
            DR5 = CR2+CI5
            DR2 = CR2-CI5
            DI5 = CI2-CR5
            DI2 = CI2+CR5
            CH(I-1,K,2) = WA1(I-2)*DR2-WA1(I-1)*DI2    
            CH(I,K,2) = WA1(I-2)*DI2+WA1(I-1)*DR2 
            CH(I-1,K,3) = WA2(I-2)*DR3-WA2(I-1)*DI3    
            CH(I,K,3) = WA2(I-2)*DI3+WA2(I-1)*DR3 
            CH(I-1,K,4) = WA3(I-2)*DR4-WA3(I-1)*DI4    
            CH(I,K,4) = WA3(I-2)*DI4+WA3(I-1)*DR4 
            CH(I-1,K,5) = WA4(I-2)*DR5-WA4(I-1)*DI5    
            CH(I,K,5) = WA4(I-2)*DI5+WA4(I-1)*DR5 
  105    CONTINUE   
  106 CONTINUE 
      RETURN   
      END 
      SUBROUTINE DRADBG(IDO,IP,L1,IDL1,CC,C1,C2,CH,CH2,WA)   
C***BEGIN PROLOGUE  DRADBG
C***REFER TO  DRFFTB 
C***ROUTINES CALLED  (NONE)   
C***END PROLOGUE  DRADBG  
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION       CH(IDO,L1,IP)          ,CC(IDO,IP,L1)          ,
     1                C1(IDO,L1,IP)          ,C2(IDL1,IP),  
     2                CH2(IDL1,IP)           ,WA(*)    
C***FIRST EXECUTABLE STATEMENT  DRADBG   
      TPI = 8.0D0*ATAN(1.0D0)  
      ARG = TPI/DBLE(IP) 
      DCP = COS(ARG)
      DSP = SIN(ARG)
      IDP2 = IDO+2  
      NBD = (IDO-1)/2    
      IPP2 = IP+2   
      IPPH = (IP+1)/2    
      IF (IDO .LT. L1) GO TO 103   
      DO 102 K=1,L1 
         DO 101 I=1,IDO  
            CH(I,K,1) = CC(I,1,K)  
  101    CONTINUE   
  102 CONTINUE 
      GO TO 106
  103 DO 105 I=1,IDO
         DO 104 K=1,L1   
            CH(I,K,1) = CC(I,1,K)  
  104    CONTINUE   
  105 CONTINUE 
  106 DO 108 J=2,IPPH    
         JC = IPP2-J
         J2 = J+J   
         DO 107 K=1,L1   
            CH(1,K,J) = CC(IDO,J2-2,K)+CC(IDO,J2-2,K)  
            CH(1,K,JC) = CC(1,J2-1,K)+CC(1,J2-1,K)
  107    CONTINUE   
  108 CONTINUE 
      IF (IDO .EQ. 1) GO TO 116    
      IF (NBD .LT. L1) GO TO 112   
      DO 111 J=2,IPPH    
         JC = IPP2-J
         DO 110 K=1,L1   
CDIR$ IVDEP    
            DO 109 I=3,IDO,2  
               IC = IDP2-I    
               CH(I-1,K,J) = CC(I-1,2*J-1,K)+CC(IC-1,2*J-2,K)    
               CH(I-1,K,JC) = CC(I-1,2*J-1,K)-CC(IC-1,2*J-2,K)   
               CH(I,K,J) = CC(I,2*J-1,K)-CC(IC,2*J-2,K)
               CH(I,K,JC) = CC(I,2*J-1,K)+CC(IC,2*J-2,K)    
  109       CONTINUE
  110    CONTINUE   
  111 CONTINUE 
      GO TO 116
  112 DO 115 J=2,IPPH    
         JC = IPP2-J
CDIR$ IVDEP    
         DO 114 I=3,IDO,2
            IC = IDP2-I  
            DO 113 K=1,L1
               CH(I-1,K,J) = CC(I-1,2*J-1,K)+CC(IC-1,2*J-2,K)    
               CH(I-1,K,JC) = CC(I-1,2*J-1,K)-CC(IC-1,2*J-2,K)   
               CH(I,K,J) = CC(I,2*J-1,K)-CC(IC,2*J-2,K)
               CH(I,K,JC) = CC(I,2*J-1,K)+CC(IC,2*J-2,K)    
  113       CONTINUE
  114    CONTINUE   
  115 CONTINUE 
  116 AR1 = 1.0D0 
      AI1 = 0.0D0 
      DO 120 L=2,IPPH    
         LC = IPP2-L
         AR1H = DCP*AR1-DSP*AI1    
         AI1 = DCP*AI1+DSP*AR1
         AR1 = AR1H 
         DO 117 IK=1,IDL1
            C2(IK,L) = CH2(IK,1)+AR1*CH2(IK,2)    
            C2(IK,LC) = AI1*CH2(IK,IP)  
  117    CONTINUE   
         DC2 = AR1  
         DS2 = AI1  
         AR2 = AR1  
         AI2 = AI1  
         DO 119 J=3,IPPH 
            JC = IPP2-J  
            AR2H = DC2*AR2-DS2*AI2 
            AI2 = DC2*AI2+DS2*AR2  
            AR2 = AR2H   
            DO 118 IK=1,IDL1  
               C2(IK,L) = C2(IK,L)+AR2*CH2(IK,J)  
               C2(IK,LC) = C2(IK,LC)+AI2*CH2(IK,JC)    
  118       CONTINUE
  119    CONTINUE   
  120 CONTINUE 
      DO 122 J=2,IPPH    
         DO 121 IK=1,IDL1
            CH2(IK,1) = CH2(IK,1)+CH2(IK,J)  
  121    CONTINUE   
  122 CONTINUE 
      DO 124 J=2,IPPH    
         JC = IPP2-J
         DO 123 K=1,L1   
            CH(1,K,J) = C1(1,K,J)-C1(1,K,JC) 
            CH(1,K,JC) = C1(1,K,J)+C1(1,K,JC)
  123    CONTINUE   
  124 CONTINUE 
      IF (IDO .EQ. 1) GO TO 132    
      IF (NBD .LT. L1) GO TO 128   
      DO 127 J=2,IPPH    
         JC = IPP2-J
         DO 126 K=1,L1   
CDIR$ IVDEP    
            DO 125 I=3,IDO,2  
               CH(I-1,K,J) = C1(I-1,K,J)-C1(I,K,JC)    
               CH(I-1,K,JC) = C1(I-1,K,J)+C1(I,K,JC)   
               CH(I,K,J) = C1(I,K,J)+C1(I-1,K,JC) 
               CH(I,K,JC) = C1(I,K,J)-C1(I-1,K,JC)
  125       CONTINUE
  126    CONTINUE   
  127 CONTINUE 
      GO TO 132
  128 DO 131 J=2,IPPH    
         JC = IPP2-J
         DO 130 I=3,IDO,2
            DO 129 K=1,L1
               CH(I-1,K,J) = C1(I-1,K,J)-C1(I,K,JC)    
               CH(I-1,K,JC) = C1(I-1,K,J)+C1(I,K,JC)   
               CH(I,K,J) = C1(I,K,J)+C1(I-1,K,JC) 
               CH(I,K,JC) = C1(I,K,J)-C1(I-1,K,JC)
  129       CONTINUE
  130    CONTINUE   
  131 CONTINUE 
  132 CONTINUE 
      IF (IDO .EQ. 1) RETURN  
      DO 133 IK=1,IDL1   
         C2(IK,1) = CH2(IK,1) 
  133 CONTINUE 
      DO 135 J=2,IP 
         DO 134 K=1,L1   
            C1(1,K,J) = CH(1,K,J)  
  134    CONTINUE   
  135 CONTINUE 
      IF (NBD .GT. L1) GO TO 139   
      IS = -IDO
      DO 138 J=2,IP 
         IS = IS+IDO
         IDIJ = IS  
         DO 137 I=3,IDO,2
            IDIJ = IDIJ+2
            DO 136 K=1,L1
               C1(I-1,K,J) = WA(IDIJ-1)*CH(I-1,K,J)-WA(IDIJ)*CH(I,K,J)
               C1(I,K,J) = WA(IDIJ-1)*CH(I,K,J)+WA(IDIJ)*CH(I-1,K,J)  
  136       CONTINUE
  137    CONTINUE   
  138 CONTINUE 
      GO TO 143
  139 IS = -IDO
      DO 142 J=2,IP 
         IS = IS+IDO
         DO 141 K=1,L1   
            IDIJ = IS    
CDIR$ IVDEP    
            DO 140 I=3,IDO,2  
               IDIJ = IDIJ+2  
               C1(I-1,K,J) = WA(IDIJ-1)*CH(I-1,K,J)-WA(IDIJ)*CH(I,K,J)
               C1(I,K,J) = WA(IDIJ-1)*CH(I,K,J)+WA(IDIJ)*CH(I-1,K,J)  
  140       CONTINUE
  141    CONTINUE   
  142 CONTINUE 
  143 RETURN   
      END 
